"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Try = void 0;
const NoSuchElementException_1 = require("../exceptions/NoSuchElementException");
class Try {
    constructor() { }
    static success(value) {
        return Try.successFactory(value);
    }
    static failure(ex) {
        return Try.failureFactory(ex);
    }
    map(f) {
        if (this.isFailure())
            return this;
        try {
            const value = this.get();
            return Try.success(f(value));
        }
        catch (e) {
            return Try.failure(e);
        }
    }
    flatMap(f) {
        if (this.isFailure())
            return this;
        try {
            const value = this.get();
            return f(value);
        }
        catch (e) {
            return Try.failure(e);
        }
    }
    static of(f) {
        try {
            return Try.success(f());
        }
        catch (e) {
            return Try.failure(e);
        }
    }
    getOrElse(fallbackValue) {
        return this.isSuccess() ? this.get() : fallbackValue;
    }
    getOrElseGet(fallbackFunction) {
        return this.isSuccess() ? this.get() : fallbackFunction(this.getCause());
    }
    getOrElseThrow(fallbackFunction) {
        if (this.isSuccess())
            return this.get();
        throw Error;
    }
    andThen(f) {
        if (this.isFailure())
            return this;
        try {
            f(this.get());
            return this;
        }
        catch (ex) {
            return Try.failure(ex);
        }
    }
    filter(predicate, fallbackFunction) {
        if (this.isFailure())
            return this;
        try {
            const value = this.get();
            if (predicate(value))
                return this;
            return Try.failure(fallbackFunction ? fallbackFunction() : new NoSuchElementException_1.NoSuchElementException("Predicate does not hold for " + value));
        }
        catch (e) {
            return Try.failure(e);
        }
    }
    filterNot(predicate, fallbackFunction) {
        return this.filter(v => !predicate(v), fallbackFunction);
    }
    peek(f) {
        if (this.isSuccess())
            f(this.get());
        return this;
    }
    recover(recoverMapping) {
        if (this.isSuccess())
            return this;
        const recoverValue = recoverMapping[this.getCause().constructor.name];
        return recoverValue ? Try.success(recoverValue) : this;
    }
    recoverWith(recoverMapping) {
        if (this.isSuccess())
            return this;
        const recoverFunction = recoverMapping[this.getCause().constructor.name];
        return recoverFunction ? Try.of(() => { return recoverFunction(this.getCause()); }) : this;
    }
    onSuccess(f) {
        if (this.isSuccess())
            f(this.get());
        return this;
    }
    onFailure(f) {
        if (this.isFailure())
            f(this.getCause());
        return this;
    }
}
exports.Try = Try;
